@@ -28,8 +28,8 @@ def multiclass_nms(multi_bboxes,
             bboxes. Default to False.
 
     Returns:
-        tuple: (bboxes, labels, indices (optional)), tensors of shape (k, 5),
-            (k), and (k). Labels are 0-based.
+        tuple: (dets, labels, indices (optional)), tensors of shape (k, 5),
+            (k), and (k). Dets are boxes with scores. Labels are 0-based.
     """
     num_classes = multi_scores.size(1) - 1
     # exclude background category
@@ -48,8 +48,10 @@ def multiclass_nms(multi_bboxes,
     scores = scores.reshape(-1)
     labels = labels.reshape(-1)
 
-    # remove low scoring boxes
-    valid_mask = scores > score_thr
+    if not torch.onnx.is_in_onnx_export():
+        # NonZero not supported  in TensorRT
+        # remove low scoring boxes
+        valid_mask = scores > score_thr
     # multiply score_factor after threshold to preserve more bboxes, improve
     # mAP by 1% for YOLOv3
     if score_factors is not None:
@@ -58,18 +60,28 @@ def multiclass_nms(multi_bboxes,
             multi_scores.size(0), num_classes)
         score_factors = score_factors.reshape(-1)
         scores = scores * score_factors
-    inds = valid_mask.nonzero(as_tuple=False).squeeze(1)
-    bboxes, scores, labels = bboxes[inds], scores[inds], labels[inds]
-    if inds.numel() == 0:
+
+    if not torch.onnx.is_in_onnx_export():
+        # NonZero not supported  in TensorRT
+        inds = valid_mask.nonzero(as_tuple=False).squeeze(1)
+        bboxes, scores, labels = bboxes[inds], scores[inds], labels[inds]
+    else:
+        # TensorRT NMS plugin has invalid output filled with -1
+        # add dummy data to make detection output correct.
+        bboxes = torch.cat([bboxes, bboxes.new_zeros(1, 4)], dim=0)
+        scores = torch.cat([scores, scores.new_zeros(1)], dim=0)
+        labels = torch.cat([labels, labels.new_zeros(1)], dim=0)
+
+    if bboxes.numel() == 0:
         if torch.onnx.is_in_onnx_export():
             raise RuntimeError('[ONNX Error] Can not record NMS '
                                'as it has not been executed this time')
+        dets = torch.cat([bboxes, scores[:, None]], -1)
         if return_inds:
-            return bboxes, labels, inds
+            return dets, labels, inds
         else:
-            return bboxes, labels
+            return dets, labels
 
-    # TODO: add size check before feed into batched_nms
     dets, keep = batched_nms(bboxes, scores, labels, nms_cfg)
 
     if max_num > 0:
@@ -111,8 +123,9 @@ def fast_nms(multi_bboxes,
             Default: -1.
 
     Returns:
-        tuple: (bboxes, labels, coefficients), tensors of shape (k, 5), (k, 1),
-            and (k, coeffs_dim). Labels are 0-based.
+        tuple: (dets, labels, coefficients), tensors of shape (k, 5), (k, 1),
+            and (k, coeffs_dim). Dets are boxes with scores.
+            Labels are 0-based.
     """
 
     scores = multi_scores[:, :-1].t()  # [#class, n]
