@@ -1,11 +1,15 @@
+import copy
+import warnings
+
 import numpy as np
 import torch
+from mmcv import ConfigDict
 from mmcv.ops import nms
 
 from ..bbox import bbox_mapping_back
 
 
-def merge_aug_proposals(aug_proposals, img_metas, rpn_test_cfg):
+def merge_aug_proposals(aug_proposals, img_metas, cfg):
     """Merge augmented proposals (multiscale, flip, etc.)
 
     Args:
@@ -19,11 +23,40 @@ def merge_aug_proposals(aug_proposals, img_metas, rpn_test_cfg):
             For details on the values of these keys see
             `mmdet/datasets/pipelines/formatting.py:Collect`.
 
-        rpn_test_cfg (dict): rpn test config.
+        cfg (dict): rpn test config.
 
     Returns:
         Tensor: shape (n, 4), proposals corresponding to original image scale.
     """
+
+    cfg = copy.deepcopy(cfg)
+
+    # deprecate arguments warning
+    if 'nms' not in cfg or 'max_num' in cfg or 'nms_thr' in cfg:
+        warnings.warn(
+            'In rpn_proposal or test_cfg, '
+            'nms_thr has been moved to a dict named nms as '
+            'iou_threshold, max_num has been renamed as max_per_img, '
+            'name of original arguments and the way to specify '
+            'iou_threshold of NMS will be deprecated.')
+    if 'nms' not in cfg:
+        cfg.nms = ConfigDict(dict(type='nms', iou_threshold=cfg.nms_thr))
+    if 'max_num' in cfg:
+        if 'max_per_img' in cfg:
+            assert cfg.max_num == cfg.max_per_img, f'You set max_num and ' \
+                f'max_per_img at the same time, but get {cfg.max_num} ' \
+                f'and {cfg.max_per_img} respectively' \
+                f'Please delete max_num which will be deprecated.'
+        else:
+            cfg.max_per_img = cfg.max_num
+    if 'nms_thr' in cfg:
+        assert cfg.nms.iou_threshold == cfg.nms_thr, f'You set ' \
+            f'iou_threshold in nms and ' \
+            f'nms_thr at the same time, but get ' \
+            f'{cfg.nms.iou_threshold} and {cfg.nms_thr}' \
+            f' respectively. Please delete the nms_thr ' \
+            f'which will be deprecated.'
+
     recovered_proposals = []
     for proposals, img_info in zip(aug_proposals, img_metas):
         img_shape = img_info['img_shape']
@@ -38,10 +71,10 @@ def merge_aug_proposals(aug_proposals, img_metas, rpn_test_cfg):
     aug_proposals = torch.cat(recovered_proposals, dim=0)
     merged_proposals, _ = nms(aug_proposals[:, :4].contiguous(),
                               aug_proposals[:, -1].contiguous(),
-                              rpn_test_cfg.nms_thr)
+                              cfg.nms.iou_threshold)
     scores = merged_proposals[:, 4]
     _, order = scores.sort(0, descending=True)
-    num = min(rpn_test_cfg.max_num, merged_proposals.shape[0])
+    num = min(cfg.max_per_img, merged_proposals.shape[0])
     order = order[:num]
     merged_proposals = merged_proposals[order, :]
     return merged_proposals
@@ -104,6 +137,9 @@ def merge_aug_masks(aug_masks, img_metas, rcnn_test_cfg, weights=None):
                 mask = mask[:, :, :, ::-1]
             elif flip_direction == 'vertical':
                 mask = mask[:, :, ::-1, :]
+            elif flip_direction == 'diagonal':
+                mask = mask[:, :, :, ::-1]
+                mask = mask[:, :, ::-1, :]
             else:
                 raise ValueError(
                     f"Invalid flipping direction '{flip_direction}'")
