@@ -9,7 +9,7 @@ from ..builder import PIPELINES
 
 
 @PIPELINES.register_module()
-class LoadImageFromFile(object):
+class LoadImageFromFile:
     """Load an image from file.
 
     Required keys are "img_prefix" and "img_info" (a dict that must contain the
@@ -110,7 +110,7 @@ class LoadImageFromWebcam(LoadImageFromFile):
 
 
 @PIPELINES.register_module()
-class LoadMultiChannelImageFromFiles(object):
+class LoadMultiChannelImageFromFiles:
     """Load multi-channel images from a list of separate channel files.
 
     Required keys are "img_prefix" and "img_info" (a dict that must contain the
@@ -193,8 +193,8 @@ class LoadMultiChannelImageFromFiles(object):
 
 
 @PIPELINES.register_module()
-class LoadAnnotations(object):
-    """Load mutiple types of annotations.
+class LoadAnnotations:
+    """Load multiple types of annotations.
 
     Args:
         with_bbox (bool): Whether to parse and load the bbox annotation.
@@ -378,14 +378,14 @@ class LoadAnnotations(object):
         repr_str += f'(with_bbox={self.with_bbox}, '
         repr_str += f'with_label={self.with_label}, '
         repr_str += f'with_mask={self.with_mask}, '
-        repr_str += f'with_seg={self.with_seg})'
-        repr_str += f'poly2mask={self.poly2mask})'
+        repr_str += f'with_seg={self.with_seg}, '
+        repr_str += f'poly2mask={self.poly2mask}, '
         repr_str += f'poly2mask={self.file_client_args})'
         return repr_str
 
 
 @PIPELINES.register_module()
-class LoadProposals(object):
+class LoadProposals:
     """Load proposal pipeline.
 
     Required key is "proposals". Updated keys are "proposals", "bbox_fields".
@@ -430,7 +430,7 @@ class LoadProposals(object):
 
 
 @PIPELINES.register_module()
-class FilterAnnotations(object):
+class FilterAnnotations:
     """Filter invalid annotations.
 
     Args:
